@@ -15,7 +15,7 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * Copyright (C) 2004 - 2008 Red Hat, Inc.
+ * Copyright (C) 2004 - 2010 Red Hat, Inc.
  * Copyright (C) 2005 - 2008 Novell, Inc.
  *
  * This applet used the GNOME Wireless Applet as a skeleton to build from.
@@ -36,7 +36,6 @@
 #include <strings.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
-#include "wireless-helper.h"
 #include <unistd.h>
 #include <sys/socket.h>
 
@@ -483,13 +482,11 @@ applet_new_menu_item_helper (NMConnection *connection,
 static gboolean
 menu_title_item_expose (GtkWidget *widget, GdkEventExpose *event)
 {
+	GtkAllocation allocation;
 	GtkStyle *style;
 	GtkWidget *label;
 	PangoFontDescription *desc;
 	cairo_t *cr;
-	GtkImageMenuItem *image_menu_item;
-	GtkImage *icon;
-	GdkPixbuf *pixbuf;
 	PangoLayout *layout;
 	int width = 0, height = 0, owidth, oheight;
 	gdouble extraheight = 0, extrawidth = 0;
@@ -500,7 +497,7 @@ menu_title_item_expose (GtkWidget *widget, GdkEventExpose *event)
 
 	label = gtk_bin_get_child (GTK_BIN (widget));
 
-	cr = gdk_cairo_create (widget->window);
+	cr = gdk_cairo_create (gtk_widget_get_window (widget));
 
 	/* The drawing area we get is the whole menu; clip the drawing to the
 	 * event area, which should just be our menu item.
@@ -513,13 +510,13 @@ menu_title_item_expose (GtkWidget *widget, GdkEventExpose *event)
 	/* We also need to reposition the cairo context so that (0, 0) is the
 	 * top-left of where we're supposed to start drawing.
 	 */
+#if GTK_CHECK_VERSION(2,18,0)
+	gtk_widget_get_allocation (widget, &allocation);
+#else
+	allocation = widget->allocation;
+#endif
 	cairo_translate (cr, widget->allocation.x, widget->allocation.y);
 
-	image_menu_item = GTK_IS_IMAGE_MENU_ITEM (widget) ? GTK_IMAGE_MENU_ITEM (widget) : NULL;
-
-	icon = image_menu_item ? GTK_IMAGE (gtk_image_menu_item_get_image (image_menu_item)) : NULL;
-	pixbuf = icon ? gtk_image_get_pixbuf (icon) : NULL;
-
 	text = gtk_label_get_text (GTK_LABEL (label));
 
 	layout = pango_cairo_create_layout (cr);
@@ -534,20 +531,6 @@ menu_title_item_expose (GtkWidget *widget, GdkEventExpose *event)
 	width = owidth / PANGO_SCALE;
 	height += oheight / PANGO_SCALE;
 
-	if (pixbuf) {
-		gdouble w = gdk_pixbuf_get_width (pixbuf);
-		gdouble h = gdk_pixbuf_get_height (pixbuf);
-
-		if (width < w)
-			width = w;
-		if (height < h) {
-			extraheight = (h - height) / 2.0;
-			height = h;
-		}
-		extrawidth = gdk_pixbuf_get_width (pixbuf) + 5.0 + 5.0;
-	}
-	width += extrawidth;
-
 	cairo_save (cr);
 
 	cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.0);
@@ -558,17 +541,6 @@ menu_title_item_expose (GtkWidget *widget, GdkEventExpose *event)
 
 	/* now the in-padding content */
 	cairo_translate (cr, xpadding , ypadding);
-	if (pixbuf) {
-		gdouble x = 0.0;
-		gdouble y = height / 2.0 - gdk_pixbuf_get_height (pixbuf) / 2.0;
-		gdouble w = gdk_pixbuf_get_width (pixbuf);
-		gdouble h = gdk_pixbuf_get_height (pixbuf);
-
-		gdk_cairo_set_source_pixbuf (cr, pixbuf, x, y);
-		cairo_rectangle (cr, x, y, w, h);
-		cairo_fill (cr);
-	}
-
 	cairo_set_source_rgb (cr, TITLE_TEXT_R, TITLE_TEXT_G, TITLE_TEXT_B);
 	cairo_move_to (cr, extrawidth, extraheight);
 	pango_cairo_show_layout (cr, layout);
@@ -589,14 +561,10 @@ applet_menu_item_create_device_item_helper (NMDevice *device,
                                             NMApplet *applet,
                                             const gchar *text)
 {
-	GtkWidget *item = gtk_image_menu_item_new_with_mnemonic (text);
-	NMADeviceClass *klass = get_device_class (device, applet);
-	GdkPixbuf *pixbuf = klass->get_device_icon ? klass->get_device_icon (device, applet) : NULL;
-	GtkWidget *image = pixbuf ? gtk_image_new_from_pixbuf (pixbuf) : NULL;
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	GtkWidget *item;
 
+	item = gtk_menu_item_new_with_mnemonic (text);
 	gtk_widget_set_sensitive (item, FALSE);
-
 	g_signal_connect (item, "expose-event", G_CALLBACK (menu_title_item_expose), NULL);
 	return item;
 }
@@ -612,6 +580,25 @@ applet_clear_notify (NMApplet *applet)
 	applet->notification = NULL;
 }
 
+static gboolean
+applet_notify_server_has_actions (void)
+{
+	gboolean has_actions = FALSE;
+	GList *server_caps, *iter;
+
+	server_caps = notify_get_server_caps();
+	for (iter = server_caps; iter; iter = g_list_next (iter)) {
+		if (!strcmp ((const char *) iter->data, NOTIFY_CAPS_ACTIONS_KEY)) {
+			has_actions = TRUE;
+			break;
+		}
+	}
+	g_list_foreach (server_caps, (GFunc) g_free, NULL);
+	g_list_free (server_caps);
+
+	return has_actions;
+}
+
 void
 applet_do_notify (NMApplet *applet,
                   NotifyUrgency urgency,
@@ -631,10 +618,8 @@ applet_do_notify (NMApplet *applet,
 	g_return_if_fail (summary != NULL);
 	g_return_if_fail (message != NULL);
 
-#if GTK_CHECK_VERSION(2, 11, 0)
 	if (!gtk_status_icon_is_embedded (applet->status_icon))
 		return;
-#endif
 
 	applet_clear_notify (applet);
 
@@ -650,7 +635,7 @@ applet_do_notify (NMApplet *applet,
 	notify_notification_set_urgency (notify, urgency);
 	notify_notification_set_timeout (notify, NOTIFY_EXPIRES_DEFAULT);
 
-	if (applet->notify_with_actions && action1) {
+	if (applet->notify_actions && action1) {
 		notify_notification_add_action (notify, action1, action1_label,
 		                                action1_cb, action1_user_data, NULL);
 	}
@@ -762,18 +747,6 @@ applet_is_any_vpn_activating (NMApplet *applet)
 	}
 	return FALSE;
 }
-static void
-save_timestamp_cb (NMSettingsConnectionInterface *connection,
-                   GError *error,
-                   gpointer user_data)
-{
-	if (error) {
-		g_warning ("Error saving connection %s timestamp: (%d) %s",
-		           nm_connection_get_path (NM_CONNECTION (connection)),
-		           error->code,
-		           error->message);
-	}
-}
 
 static void
 update_connection_timestamp (NMActiveConnection *active,
@@ -788,14 +761,15 @@ update_connection_timestamp (NMActiveConnection *active,
 
 	gconf_connection = nm_settings_interface_get_connection_by_path (NM_SETTINGS_INTERFACE (applet->gconf_settings),
 	                                                                 nm_connection_get_path (connection));
-	if (!gconf_connection)
+	if (!gconf_connection || !NMA_IS_GCONF_CONNECTION (gconf_connection))
 		return;
 
 	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
 	g_assert (s_con);
 
 	g_object_set (s_con, NM_SETTING_CONNECTION_TIMESTAMP, (guint64) time (NULL), NULL);
-	nm_settings_connection_interface_update (gconf_connection, save_timestamp_cb, NULL);
+	/* Ignore secrets since we're just updating the timestamp */
+	nma_gconf_connection_update (NMA_GCONF_CONNECTION (gconf_connection), TRUE);
 }
 
 static char *
@@ -1090,7 +1064,7 @@ applet_get_first_active_vpn_connection (NMApplet *applet,
 /*
  * nma_menu_disconnect_vpn_item_activate
  *
- * Signal function called when user clicks "Disconnect VPN..."
+ * Signal function called when user clicks "Disconnect VPN"
  *
  */
 static void
@@ -1318,8 +1292,12 @@ nma_menu_device_get_menu_item (NMDevice *device,
 	GtkWidget *item = NULL;
 	gboolean managed = TRUE;
 
-	if (!unavailable_msg)
-		unavailable_msg = _("device not ready");
+	if (!unavailable_msg) {
+		if (nm_device_get_firmware_missing (device))
+			unavailable_msg = _("device not ready (firmware missing)");
+		else
+			unavailable_msg = _("device not ready");
+	}
 
 	switch (nm_device_get_state (device)) {
 	case NM_DEVICE_STATE_UNKNOWN:
@@ -1549,7 +1527,7 @@ nma_menu_add_vpn_submenu (GtkWidget *menu, NMApplet *applet)
 	g_signal_connect (item, "activate", G_CALLBACK (nma_menu_configure_vpn_item_activate), applet);
 	gtk_menu_shell_append (GTK_MENU_SHELL (vpn_menu), GTK_WIDGET (item));
 
-	item = GTK_MENU_ITEM (gtk_menu_item_new_with_mnemonic (_("_Disconnect VPN...")));
+	item = GTK_MENU_ITEM (gtk_menu_item_new_with_mnemonic (_("_Disconnect VPN")));
 	g_signal_connect (item, "activate", G_CALLBACK (nma_menu_disconnect_vpn_item_activate), applet);
 	gtk_menu_shell_append (GTK_MENU_SHELL (vpn_menu), GTK_WIDGET (item));
 	if (num_vpn_active == 0)
@@ -1589,7 +1567,7 @@ nma_set_networking_enabled_cb (GtkWidget *widget, NMApplet *applet)
 	g_return_if_fail (applet != NULL);
 
 	state = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget));
-	nm_client_sleep (applet->nm_client, !state);
+	nm_client_networking_set_enabled (applet->nm_client, state);
 }
 
 
@@ -1716,6 +1694,14 @@ static gboolean nma_menu_clear (NMApplet *applet)
 	return FALSE;
 }
 
+static gboolean
+is_permission_yes (NMApplet *applet, NMClientPermission perm)
+{
+	if (   applet->permissions[perm] == NM_CLIENT_PERMISSION_RESULT_YES
+	    || applet->permissions[perm] == NM_CLIENT_PERMISSION_RESULT_AUTH)
+		return TRUE;
+	return FALSE;
+}
 
 /*
  * nma_context_menu_update
@@ -1725,6 +1711,7 @@ static void
 nma_context_menu_update (NMApplet *applet)
 {
 	NMState state;
+	gboolean net_enabled = TRUE;
 	gboolean have_wireless = FALSE;
 	gboolean have_wwan = FALSE;
 	gboolean wireless_hw_enabled;
@@ -1742,10 +1729,13 @@ nma_context_menu_update (NMApplet *applet)
 	/* Enabled Networking */
 	g_signal_handler_block (G_OBJECT (applet->networking_enabled_item),
 	                        applet->networking_enabled_toggled_id);
+	net_enabled = nm_client_networking_get_enabled (applet->nm_client);
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (applet->networking_enabled_item),
-	                                state != NM_STATE_ASLEEP);
+	                                net_enabled && (state != NM_STATE_ASLEEP));
 	g_signal_handler_unblock (G_OBJECT (applet->networking_enabled_item),
 	                          applet->networking_enabled_toggled_id);
+	gtk_widget_set_sensitive (applet->networking_enabled_item,
+	                          is_permission_yes (applet, NM_CLIENT_PERMISSION_ENABLE_DISABLE_NETWORK));
 
 	/* Enabled Wireless */
 	g_signal_handler_block (G_OBJECT (applet->wifi_enabled_item),
@@ -1757,7 +1747,7 @@ nma_context_menu_update (NMApplet *applet)
 
 	wireless_hw_enabled = nm_client_wireless_hardware_get_enabled (applet->nm_client);
 	gtk_widget_set_sensitive (GTK_WIDGET (applet->wifi_enabled_item),
-	                          wireless_hw_enabled);
+	                          wireless_hw_enabled && is_permission_yes (applet, NM_CLIENT_PERMISSION_ENABLE_DISABLE_WIFI));
 
 	/* Enabled Mobile Broadband */
 	g_signal_handler_block (G_OBJECT (applet->wwan_enabled_item),
@@ -1769,7 +1759,7 @@ nma_context_menu_update (NMApplet *applet)
 
 	wwan_hw_enabled = nm_client_wwan_hardware_get_enabled (applet->nm_client);
 	gtk_widget_set_sensitive (GTK_WIDGET (applet->wwan_enabled_item),
-	                          wwan_hw_enabled);
+	                          wwan_hw_enabled && is_permission_yes (applet, NM_CLIENT_PERMISSION_ENABLE_DISABLE_WWAN));
 
 	/* Enabled notifications */
 	g_signal_handler_block (G_OBJECT (applet->notifications_enabled_item),
@@ -1968,6 +1958,7 @@ foo_set_icon (NMApplet *applet, GdkPixbuf *pixbuf, guint32 layer)
 		applet->icon_layers[layer] = g_object_ref (pixbuf);
 
 	if (!applet->icon_layers[0]) {
+		nma_icon_check_and_load ("nm-no-connection", &applet->no_connection_icon, applet);
 		pixbuf = g_object_ref (applet->no_connection_icon);
 	} else {
 		pixbuf = gdk_pixbuf_copy (applet->icon_layers[0]);
@@ -2005,9 +1996,14 @@ applet_get_exported_connection_for_device (NMDevice *device, NMApplet *applet)
 		const GPtrArray *devices;
 
 		active = g_ptr_array_index (active_connections, i);
+		if (!active)
+			continue;
+
 		devices = nm_active_connection_get_devices (active);
 		service_name = nm_active_connection_get_service_name (active);
 		connection_path = nm_active_connection_get_connection (active);
+		if (!devices || !service_name || !connection_path)
+			continue;
 
 		if (strcmp (service_name, NM_DBUS_SERVICE_USER_SETTINGS) != 0)
 			continue;
@@ -2174,6 +2170,18 @@ foo_active_connections_changed_cb (NMClient *client,
 	applet_schedule_update_icon (applet);
 }
 
+static void
+foo_manager_permission_changed (NMClient *client,
+                                NMClientPermission permission,
+                                NMClientPermissionResult result,
+                                gpointer user_data)
+{
+	NMApplet *applet = NM_APPLET (user_data);
+
+	if (permission <= NM_CLIENT_PERMISSION_LAST)
+		applet->permissions[permission] = result;
+}
+
 static gboolean
 foo_set_initial_state (gpointer data)
 {
@@ -2212,6 +2220,15 @@ foo_client_setup (NMApplet *applet)
 	                  G_CALLBACK (foo_manager_running_cb),
 	                  applet);
 
+	g_signal_connect (applet->nm_client, "permission-changed",
+	                  G_CALLBACK (foo_manager_permission_changed),
+	                  applet);
+	/* Initialize permissions - the initial 'permission-changed' signal is emitted from NMClient constructor, and thus not caught */
+	applet->permissions[NM_CLIENT_PERMISSION_ENABLE_DISABLE_NETWORK] = nm_client_get_permission_result (applet->nm_client, NM_CLIENT_PERMISSION_ENABLE_DISABLE_NETWORK);
+	applet->permissions[NM_CLIENT_PERMISSION_ENABLE_DISABLE_WIFI] = nm_client_get_permission_result (applet->nm_client, NM_CLIENT_PERMISSION_ENABLE_DISABLE_WIFI);
+	applet->permissions[NM_CLIENT_PERMISSION_ENABLE_DISABLE_WWAN] = nm_client_get_permission_result (applet->nm_client, NM_CLIENT_PERMISSION_ENABLE_DISABLE_WWAN);
+	applet->permissions[NM_CLIENT_PERMISSION_USE_USER_CONNECTIONS] = nm_client_get_permission_result (applet->nm_client, NM_CLIENT_PERMISSION_USE_USER_CONNECTIONS);
+
 	if (nm_client_get_manager_running (applet->nm_client))
 		g_idle_add (foo_set_initial_state, applet);
 }
@@ -2238,6 +2255,18 @@ applet_common_get_device_icon (NMDeviceState state, NMApplet *applet)
 	}
 
 	if (stage >= 0) {
+		int i, j;
+
+		for (i = 0; i < NUM_CONNECTING_STAGES; i++) {
+			for (j = 0; j < NUM_CONNECTING_FRAMES; j++) {
+				char *name;
+
+				name = g_strdup_printf ("nm-stage%02d-connecting%02d", i+1, j+1);
+				nma_icon_check_and_load (name, &applet->network_connecting_icons[i][j], applet);
+				g_free (name);
+			}
+		}
+
 		pixbuf = applet->network_connecting_icons[stage][applet->animation_step];
 		applet->animation_step++;
 		if (applet->animation_step >= NUM_CONNECTING_FRAMES)
@@ -2398,11 +2427,11 @@ applet_update_icon (gpointer user_data)
 	switch (state) {
 	case NM_STATE_UNKNOWN:
 	case NM_STATE_ASLEEP:
-		pixbuf = applet->no_connection_icon;
+		pixbuf = nma_icon_check_and_load ("nm-no-connection", &applet->no_connection_icon, applet);
 		dev_tip = g_strdup (_("Networking disabled"));
 		break;
 	case NM_STATE_DISCONNECTED:
-		pixbuf = applet->no_connection_icon;
+		pixbuf = nma_icon_check_and_load ("nm-no-connection", &applet->no_connection_icon, applet);
 		dev_tip = g_strdup (_("No network connection"));
 		break;
 	default:
@@ -2416,14 +2445,24 @@ applet_update_icon (gpointer user_data)
 	pixbuf = NULL;
 	active_vpn = applet_get_first_active_vpn_connection (applet, &vpn_state);
 	if (active_vpn) {
+		int i;
+
 		switch (vpn_state) {
 		case NM_VPN_CONNECTION_STATE_ACTIVATED:
-			pixbuf = applet->vpn_lock_icon;
+			pixbuf = nma_icon_check_and_load ("nm-vpn-active-lock", &applet->vpn_lock_icon, applet);
 			break;
 		case NM_VPN_CONNECTION_STATE_PREPARE:
 		case NM_VPN_CONNECTION_STATE_NEED_AUTH:
 		case NM_VPN_CONNECTION_STATE_CONNECT:
 		case NM_VPN_CONNECTION_STATE_IP_CONFIG_GET:
+			for (i = 0; i < NUM_VPN_CONNECTING_FRAMES; i++) {
+				char *name;
+
+				name = g_strdup_printf ("nm-vpn-connecting%02d", i+1);
+				nma_icon_check_and_load (name, &applet->vpn_connecting_icons[i], applet);
+				g_free (name);
+			}
+
 			pixbuf = applet->vpn_connecting_icons[applet->animation_step];
 			applet->animation_step++;
 			if (applet->animation_step >= NUM_VPN_CONNECTING_FRAMES)
@@ -2620,7 +2659,7 @@ periodic_update_active_connection_timestamps (gpointer user_data)
 
 		path = nm_active_connection_get_connection (active);
 		connection = nm_settings_interface_get_connection_by_path (NM_SETTINGS_INTERFACE (applet->gconf_settings), path);
-		if (!connection)
+		if (!connection || !NMA_IS_GCONF_CONNECTION (connection))
 			continue;
 
 		devices = nm_active_connection_get_devices (active);
@@ -2640,7 +2679,8 @@ periodic_update_active_connection_timestamps (gpointer user_data)
 				g_assert (s_con);
 
 				g_object_set (s_con, NM_SETTING_CONNECTION_TIMESTAMP, (guint64) time (NULL), NULL);
-				nm_settings_connection_interface_update (connection, save_timestamp_cb, NULL);
+				/* Ignore secrets since we're just updating the timestamp */
+				nma_gconf_connection_update (NMA_GCONF_CONNECTION (connection), TRUE);
 				break;
 			}
 		}
@@ -2651,117 +2691,119 @@ periodic_update_active_connection_timestamps (gpointer user_data)
 
 /*****************************************************************************/
 
-#define CLEAR_ICON(x) \
-	if (x) { \
-		g_object_unref (x); \
-		x = NULL; \
+static void
+nma_clear_icon (GdkPixbuf **icon, NMApplet *applet)
+{
+	g_return_if_fail (icon != NULL);
+	g_return_if_fail (applet != NULL);
+
+	if (*icon && (*icon != applet->fallback_icon)) {
+		g_object_unref (*icon);
+		*icon = NULL;
 	}
+}
 
 static void nma_icons_free (NMApplet *applet)
 {
 	int i, j;
 
-	if (!applet->icons_loaded)
-		return;
-
 	for (i = 0; i <= ICON_LAYER_MAX; i++)
-		CLEAR_ICON(applet->icon_layers[i]);
+		nma_clear_icon (&applet->icon_layers[i], applet);
 
-	CLEAR_ICON(applet->no_connection_icon);
-	CLEAR_ICON(applet->wired_icon);
-	CLEAR_ICON(applet->adhoc_icon);
-	CLEAR_ICON(applet->wwan_icon);
-	CLEAR_ICON(applet->vpn_lock_icon);
-	CLEAR_ICON(applet->wireless_00_icon);
-	CLEAR_ICON(applet->wireless_25_icon);
-	CLEAR_ICON(applet->wireless_50_icon);
-	CLEAR_ICON(applet->wireless_75_icon);
-	CLEAR_ICON(applet->wireless_100_icon);
-	CLEAR_ICON(applet->secure_lock_icon);
+	nma_clear_icon (&applet->no_connection_icon, applet);
+	nma_clear_icon (&applet->wired_icon, applet);
+	nma_clear_icon (&applet->adhoc_icon, applet);
+	nma_clear_icon (&applet->wwan_icon, applet);
+	nma_clear_icon (&applet->wwan_tower_icon, applet);
+	nma_clear_icon (&applet->vpn_lock_icon, applet);
+	nma_clear_icon (&applet->wireless_00_icon, applet);
+	nma_clear_icon (&applet->wireless_25_icon, applet);
+	nma_clear_icon (&applet->wireless_50_icon, applet);
+	nma_clear_icon (&applet->wireless_75_icon, applet);
+	nma_clear_icon (&applet->wireless_100_icon, applet);
+	nma_clear_icon (&applet->secure_lock_icon, applet);
+
+	nma_clear_icon (&applet->mb_tech_1x_icon, applet);
+	nma_clear_icon (&applet->mb_tech_evdo_icon, applet);
+	nma_clear_icon (&applet->mb_tech_gprs_icon, applet);
+	nma_clear_icon (&applet->mb_tech_edge_icon, applet);
+	nma_clear_icon (&applet->mb_tech_umts_icon, applet);
+	nma_clear_icon (&applet->mb_tech_hspa_icon, applet);
+	nma_clear_icon (&applet->mb_roaming_icon, applet);
+	nma_clear_icon (&applet->mb_tech_3g_icon, applet);
 
 	for (i = 0; i < NUM_CONNECTING_STAGES; i++) {
 		for (j = 0; j < NUM_CONNECTING_FRAMES; j++)
-			CLEAR_ICON(applet->network_connecting_icons[i][j]);
+			nma_clear_icon (&applet->network_connecting_icons[i][j], applet);
 	}
 
 	for (i = 0; i < NUM_VPN_CONNECTING_FRAMES; i++)
-		CLEAR_ICON(applet->vpn_connecting_icons[i]);
+		nma_clear_icon (&applet->vpn_connecting_icons[i], applet);
 
 	for (i = 0; i <= ICON_LAYER_MAX; i++)
-		CLEAR_ICON(applet->icon_layers[i]);
-
-	applet->icons_loaded = FALSE;
+		nma_clear_icon (&applet->icon_layers[i], applet);
 }
 
-#define ICON_LOAD(icon, name) \
-	{ \
-		icon = gtk_icon_theme_load_icon (applet->icon_theme, name, applet->icon_size, 0, &err); \
-		if (icon == NULL) { \
-			g_warning ("Icon %s missing: %s", name, \
-			           (err && err->message) ? err->message : "unknown"); \
-			g_clear_error (&err); \
-			goto out; \
-		} \
+GdkPixbuf *
+nma_icon_check_and_load (const char *name, GdkPixbuf **icon, NMApplet *applet)
+{
+	GError *error = NULL;
+
+	g_return_val_if_fail (name != NULL, NULL);
+	g_return_val_if_fail (icon != NULL, NULL);
+	g_return_val_if_fail (applet != NULL, NULL);
+
+	/* icon already loaded successfully */
+	if (*icon && (*icon != applet->fallback_icon))
+		return *icon;
+
+	/* Try to load the icon; if the load fails, log the problem, and set
+	 * the icon to the fallback icon if requested.
+	 */
+	*icon = gtk_icon_theme_load_icon (applet->icon_theme, name, applet->icon_size, 0, &error);
+	if (!*icon) {
+		g_warning ("Icon %s missing: (%d) %s",
+		           name,
+		           error ? error->code : -1,
+			       (error && error->message) ? error->message : "(unknown)");
+		g_clear_error (&error);
+
+		*icon = applet->fallback_icon;
 	}
+	return *icon;
+}
+
+#define FALLBACK_ICON_NAME "gtk-dialog-error"
 
 static gboolean
-nma_icons_load (NMApplet *applet)
+nma_icons_reload (NMApplet *applet)
 {
-	int i, j;
-	GError *err = NULL;
+	GError *error = NULL;
 
-	g_return_val_if_fail (!applet->icons_loaded, FALSE);
 	g_return_val_if_fail (applet->icon_size > 0, FALSE);
 
-	ICON_LOAD(applet->no_connection_icon, "nm-no-connection");
-	ICON_LOAD(applet->wired_icon, "nm-device-wired");
-	ICON_LOAD(applet->adhoc_icon, "nm-adhoc");
-	ICON_LOAD(applet->wwan_icon, "nm-device-wwan");
-	ICON_LOAD(applet->vpn_lock_icon, "nm-vpn-active-lock");
+	nma_icons_free (applet);
 
-	ICON_LOAD(applet->wireless_00_icon, "nm-signal-00");
-	ICON_LOAD(applet->wireless_25_icon, "nm-signal-25");
-	ICON_LOAD(applet->wireless_50_icon, "nm-signal-50");
-	ICON_LOAD(applet->wireless_75_icon, "nm-signal-75");
-	ICON_LOAD(applet->wireless_100_icon, "nm-signal-100");
-	ICON_LOAD(applet->secure_lock_icon, "nm-secure-lock");
-
-	for (i = 0; i < NUM_CONNECTING_STAGES; i++) {
-		for (j = 0; j < NUM_CONNECTING_FRAMES; j++) {
-			char *name;
-
-			name = g_strdup_printf ("nm-stage%02d-connecting%02d", i+1, j+1);
-			ICON_LOAD(applet->network_connecting_icons[i][j], name);
-			g_free (name);
-		}
+	applet->fallback_icon = gtk_icon_theme_load_icon (applet->icon_theme,
+	                                                  FALLBACK_ICON_NAME,
+	                                                  applet->icon_size, 0,
+	                                                  &error);
+	if (!applet->fallback_icon) {
+		g_warning ("Fallback icon '%s' missing: (%d) %s",
+		           FALLBACK_ICON_NAME,
+		           error ? error->code : -1,
+			       (error && error->message) ? error->message : "(unknown)");
+		g_clear_error (&error);
+		/* Die if we can't get even a fallback icon */
+		g_assert (applet->fallback_icon);
 	}
 
-	for (i = 0; i < NUM_VPN_CONNECTING_FRAMES; i++) {
-		char *name;
-
-		name = g_strdup_printf ("nm-vpn-connecting%02d", i+1);
-		ICON_LOAD(applet->vpn_connecting_icons[i], name);
-		g_free (name);
-	}
-
-	applet->icons_loaded = TRUE;
-
-out:
-	if (!applet->icons_loaded) {
-		GtkWidget *dialog;
-
-		dialog = applet_warning_dialog_show (_("The NetworkManager applet could not find some required resources.  It cannot continue.\n"));
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		g_main_loop_quit (applet->loop);
-	}
-
-	return applet->icons_loaded;
+	return TRUE;
 }
 
 static void nma_icon_theme_changed (GtkIconTheme *icon_theme, NMApplet *applet)
 {
-	nma_icons_free (applet);
-	nma_icons_load (applet);
+	nma_icons_reload (applet);
 }
 
 static void nma_icons_init (NMApplet *applet)
@@ -2776,12 +2818,8 @@ static void nma_icons_init (NMApplet *applet)
 		g_object_unref (G_OBJECT (applet->icon_theme));
 	}
 
-#if GTK_CHECK_VERSION(2, 11, 0)
 	screen = gtk_status_icon_get_screen (applet->status_icon);
-#else
-	screen = gdk_screen_get_default ();
-#endif /* gtk 2.11.0 */
-	
+	g_assert (screen);
 	applet->icon_theme = gtk_icon_theme_get_for_screen (screen);
 
 	/* If not done yet, append our search path */
@@ -2816,8 +2854,7 @@ status_icon_size_changed_cb (GtkStatusIcon *icon,
 	 */
 	applet->icon_size = MAX (16, size);
 
-	nma_icons_free (applet);
-	nma_icons_load (applet);
+	nma_icons_reload (applet);
 
 	applet_schedule_update_icon (applet);
 
@@ -2890,9 +2927,12 @@ applet_pre_keyring_callback (gpointer user_data)
 	NMApplet *applet = NM_APPLET (user_data);
 	GdkScreen *screen;
 	GdkDisplay *display;
+	GdkWindow *window = NULL;
 
-	if (applet->menu && applet->menu->window) {
-		screen = gdk_drawable_get_screen (applet->menu->window);
+	if (applet->menu)
+		window = gtk_widget_get_window (applet->menu);
+	if (window) {
+		screen = gdk_drawable_get_screen (window);
 		display = gdk_screen_get_display (screen);
 		g_object_ref (display);
 
@@ -2909,8 +2949,11 @@ applet_pre_keyring_callback (gpointer user_data)
 		g_object_unref (display);
 	}
 
-	if (applet->context_menu && applet->context_menu->window) {
-		screen = gdk_drawable_get_screen (applet->context_menu->window);
+	window = NULL;
+	if (applet->context_menu)
+		window = gtk_widget_get_window (applet->context_menu);
+	if (window) {
+		screen = gdk_drawable_get_screen (window);
 		display = gdk_screen_get_display (screen);
 		g_object_ref (display);
 
@@ -2934,6 +2977,15 @@ exit_cb (GObject *ignored, gpointer user_data)
 	g_main_loop_quit (applet->loop);
 }
 
+static void
+applet_embedded_cb (GObject *object, GParamSpec *pspec, gpointer user_data)
+{
+	gboolean embedded = gtk_status_icon_is_embedded (GTK_STATUS_ICON (object));
+
+	g_message ("applet now %s the notification area",
+	           embedded ? "embedded in" : "removed from");
+}
+
 static GObject *
 constructor (GType type,
              guint n_props,
@@ -2941,7 +2993,6 @@ constructor (GType type,
 {
 	NMApplet *applet;
 	AppletDBusManager *dbus_mgr;
-	GList *server_caps, *iter;
 
 	applet = NM_APPLET (G_OBJECT_CLASS (nma_parent_class)->constructor (type, n_props, construct_props));
 
@@ -2964,6 +3015,12 @@ constructor (GType type,
 	if (!applet->gconf_client)
 		goto error;
 
+	/* Note that we don't care about change notifications for prefs values... */
+	gconf_client_add_dir (applet->gconf_client,
+	                      APPLET_PREFS_PATH,
+	                      GCONF_CLIENT_PRELOAD_ONELEVEL,
+	                      NULL);
+
 	/* Load pixmaps and create applet widgets */
 	if (!setup_widgets (applet))
 		goto error;
@@ -2972,16 +3029,6 @@ constructor (GType type,
 	if (!notify_is_initted ())
 		notify_init ("NetworkManager");
 
-	server_caps = notify_get_server_caps();
-	applet->notify_with_actions = FALSE;
-	for (iter = server_caps; iter; iter = g_list_next (iter)) {
-		if (!strcmp ((const char *) iter->data, NOTIFY_CAPS_ACTIONS_KEY))
-			applet->notify_with_actions = TRUE;
-	}
-
-	g_list_foreach (server_caps, (GFunc) g_free, NULL);
-	g_list_free (server_caps);
-
 	dbus_mgr = applet_dbus_manager_get ();
 	if (dbus_mgr == NULL) {
 		nm_warning ("Couldn't initialize the D-Bus manager.");
@@ -3029,6 +3076,15 @@ constructor (GType type,
 
 	nm_gconf_set_pre_keyring_callback (applet_pre_keyring_callback, applet);
 
+	/* Track embedding to help debug issues where user has removed the
+	 * notification area applet from the panel, and thus nm-applet too.
+	 */
+	g_signal_connect (applet->status_icon, "notify::embedded",
+	                  G_CALLBACK (applet_embedded_cb), NULL);
+	applet_embedded_cb (G_OBJECT (applet->status_icon), NULL, NULL);
+
+	applet->notify_actions = applet_notify_server_has_actions ();
+
 	return G_OBJECT (applet);
 
 error:
@@ -3066,8 +3122,12 @@ static void finalize (GObject *object)
 	if (applet->info_dialog_xml)
 		g_object_unref (applet->info_dialog_xml);
 
-	if (applet->gconf_client)
+	if (applet->gconf_client) {
+		gconf_client_remove_dir (applet->gconf_client,
+		                         APPLET_PREFS_PATH,
+		                         NULL);
 		g_object_unref (applet->gconf_client);
+	}
 
 	if (applet->status_icon)
 		g_object_unref (applet->status_icon);
@@ -3075,6 +3135,9 @@ static void finalize (GObject *object)
 	if (applet->nm_client)
 		g_object_unref (applet->nm_client);
 
+	if (applet->fallback_icon)
+		g_object_unref (applet->fallback_icon);
+
 	if (applet->gconf_settings) {
 		g_object_unref (applet->gconf_settings);
 		applet->gconf_settings = NULL;
